/*
 * Copyright (C) 2008 Carl Worth
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/ .
 *
 * Author: Carl Worth <cworth@cworth.org>
 */

#ifndef DVONN_BOARD_H
#define DVONN_BOARD_H

typedef int dvonn_bool_t;

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE  1
#endif

typedef enum {
    DVONN_PLAYER_BLACK,
    DVONN_PLAYER_WHITE
} dvonn_player_t;

typedef enum {
    DVONN_CELL_BLACK = DVONN_PLAYER_BLACK,
    DVONN_CELL_WHITE = DVONN_PLAYER_WHITE,
    DVONN_CELL_RED,
    DVONN_CELL_EMPTY,
    DVONN_CELL_INVALID
} dvonn_cell_type_t;

typedef struct {
    dvonn_cell_type_t type;
    int height;
    dvonn_bool_t contains_red;
} dvonn_cell_t;

#define DVONN_BOARD_X_SIZE 11
#define DVONN_BOARD_Y_SIZE 5

typedef enum {
    DVONN_PHASE_PLACEMENT,
    DVONN_PHASE_MOVEMENT,
    DVONN_PHASE_GAME_OVER
} dvonn_phase_t;

typedef struct {
    dvonn_cell_t cells[DVONN_BOARD_X_SIZE][DVONN_BOARD_Y_SIZE];
    dvonn_phase_t phase;
    dvonn_player_t player;
    int moves;
    int score[2]; /* index by dvonn_player_t */
} dvonn_board_t;

/* Initialize a board for a new game of DVONN. */
void
dvonn_board_init (dvonn_board_t *board);

/* Place a piece at (x,y) where (0,0) is at the upper-left corner of
 * the board. Returns TRUE if the move is legal and is performed. If
 * the move is not legal this function returns FALSE, no change will
 * be performed on the board, and *error will be set to a string
 * describing why the move is illegal.*/
int
dvonn_board_place (dvonn_board_t *board,
		   int x, int y,
		   char **error);

/* Move a piece from (x1,y1) to (x2,y2) where (0,0) is at the
 * upper-left corner of the board. Returns TRUE if the move is legal
 * and is performed. If the move is not legal this function returns
 * FALSE, no change will be performed on the board, and *error will be
 * set to a string describing why the move is illegal.*/
int
dvonn_board_move (dvonn_board_t *board,
		int x1, int y1,
		int x2, int y2,
		char **error);

/* Is the cell at (x,y) occupied by a piece. Returns FALSE for all
 * out-of-bounds coordinates. */
dvonn_bool_t
dvonn_board_cell_occupied (dvonn_board_t *board,
			   int x, int y);

/* Is the cell at (x,y) occupied by a piece (or stack) with the given
 * player's piece at the top. Returns FALSE for all out-of-bounds
 * coordinates. */
dvonn_bool_t
dvonn_board_cell_owned_by (dvonn_board_t *board,
			   int x, int y, dvonn_player_t player);

/* Is the cell at (x,y) surrounded by other pieces, (such that it is
 * not legal for a piece at (x,y) to move. */
dvonn_bool_t
dvonn_board_cell_surrounded (dvonn_board_t *board,
			     int x, int y);


#endif
